LoadDataset <- function(path, title="Unknown dataset", csv = FALSE){
  if(csv){
    X <- read.csv(path, header = FALSE)
  }
  else{
    X <- read.table(path) 
  }
  names(X) <- c("x", "y", "label")
  plot(X$x, X$y, col=X$label, xlab = "x", ylab = "y", main = title)
  return(X)
}

CalcSimMatrix <- function(points, sigma = 0.5){
  D <- dist(points, method = "euclidean") #Euclidean distance matrix
  D <- as.matrix(D)
  D <- D^2
  S <- exp(-D / (2 * sigma^2)) #Gaussian similarity function
  return(S)
}

BuildEpsilonGraph <- function(D, e = 0.2){
  #epsion graph
  ### ADD YOUR CODE HERE!
}

BuildDirectedKNNGraph <- function(D, k = 5){
  #directed graph
  ### ADD YOUR CODE HERE!
}

BuildUndirectedKNNGraph <- function(D, k = 5, mutual = FALSE){
  #Undirected graph
  #mutual TRUE
  #mutual FALSE
  ### ADD YOUR CODE HERE!
}

CalcLaplacian <- function(W){
  #Laplacian matrix
  ### ADD YOUR CODE HERE!
}

PlotConnectedGraph <- function(data, dist){
  plot(data$x, data$y, col=data$label, xlab = "x", ylab = "y", main = "Connectivity graph")
  indecesx <- which(dist > 0, arr.ind = TRUE)
  for(i in 1:nrow(indecesx)){
    lines(data[indecesx[i,],])
  }
}

#Get accuracy
Purity <- function(clusters, labels){
  #clusters - vector of values determining the class
  #labels
  correct <- max(sum(clusters == labels), sum(clusters != labels))
  return(correct/length(clusters))
}

#Dataset
mydataset <- LoadDataset("jain.txt", "A.K. Jain's Toy problem")
#mydataset <- LoadDataset("spiral.txt", "Spiral dataset")


#Try K-means
res <- kmeans(mydataset, centers = ?)
plot(mydataset$x, mydataset$y, col=res$cluster, main = "Spiral dataset - Kmeans", xlab = "x", ylab = "y")
Purity(res$cluster, mydataset$label)

#Calculate Similarity matrix
d <- CalcSimMatrix(mydataset[,1:2], sigma = ?)

#Epsilon Graph
#similarityGraph <- BuildEpsilonGraph(d, e = ?)

#Directed Graph
#similarityGraph <- BuildDirectedKNNGraph(d, k = ?)

#Undirected nonmutual/mutual Graph
similarityGraph <- BuildUndirectedKNNGraph(d, k = ?, mutual = TRUE)

PlotConnectedGraph(mydataset, similarityGraph)

#laplacian matrix
laplacian <- CalcLaplacian(similarityGraph)

#Compute eigenvectors
eigenVectors <- eigen(laplacian)
#Get the smallest k eigenvectors
finalEigenvector <- eigenVectors$vectors[,c(length(eigenVectors$values), length(eigenVectors$values)-1)]

#Kmeans
resSpectral <- kmeans(centers = ?, finalEigenvector, nstart = 20, iter.max = 100)
#Plot data
plot(mydataset$x, mydataset$y, col=resSpectral$cluster, main = "Spiral dataset - Spectral Clustering", xlab = "x", ylab = "y")
Purity(resSpectral$cluster, mydataset$label)
